/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.stat.handler;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;
import org.xml.sax.SAXException;

import benten.cat.stat.CatStatPlugin;
import benten.cat.stat.views.AbstractStatView;
import benten.core.io.ResourceFile;
import benten.core.io.XliffFileFilter;
import benten.ui.UiPlugin;
import benten.ui.UiStatus;

/**
 * 翻訳統計取得ハンドラー抽象クラスです。
 * @param <T> モデルの型
 */
public abstract class AbstractStatHandler<T> extends AbstractHandler implements IHandler {

	private IProgressMonitor monitor;

	final public Object execute(final ExecutionEvent event) throws ExecutionException {
		final File selectionFile = getSelectionFile(event);
		if (selectionFile == null) {
			final UiStatus s = new UiStatus(IStatus.INFO, "処理対象となるファイルまたはディレクトリーを選択してください。");
			UiPlugin.openDialog("処理対象を選択してください", s);
			return null;
		}
		final Job job = new Job(getJobMessage()) {
			@Override
			protected IStatus run(IProgressMonitor monitor) {
				AbstractStatHandler.this.monitor = monitor;
				monitor.beginTask(null, IProgressMonitor.UNKNOWN);
				final List<T> models = new LinkedList<T>();

				// 処理実行
				explore(selectionFile, models);

				if (monitor.isCanceled()) {
					return Status.CANCEL_STATUS;
				}
				monitor.done();
				if (models.size() == 0) {
					Display.getDefault().asyncExec(new Runnable() {
						public void run() {
							final UiStatus s = new UiStatus(IStatus.INFO, "処理対象となるファイルがありませんでした。");
							UiPlugin.openDialog("処理対象がありません", s);
						}
					});
					return Status.CANCEL_STATUS;
				}
				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						setViewInput(event, models);
					}
				});
				return Status.OK_STATUS;
			}
		};
		job.schedule();
		return null;
	}

	private ResourceFile getSelectionFile(ExecutionEvent event) {
		final ISelection sel = HandlerUtil.getCurrentSelection(event);
		if (sel instanceof IStructuredSelection) {
			final IStructuredSelection selection = (IStructuredSelection) sel;
			final Object o = selection.getFirstElement();
			if (o != null && o instanceof IResource) {
				final IResource resource = (IResource) o;
				return new ResourceFile(resource);
			}
		}
		return null;
	}

	protected String getJobMessage() {
		return "翻訳統計取得中...";
	}

	abstract protected T processFile(File file) throws IOException, SAXException;

	protected void explore(File file, List<T> models) {
		if (file.isDirectory()) {
			for (final File f : file.listFiles()) {

				// このメソッドを再帰呼び出し
				explore(f, models);
			}
		} else {
			if (XliffFileFilter.INSTANCE.accept(file, file.getName())) {
				try {
					if (monitor.isCanceled()) {
						return;
					}
					CatStatPlugin.log("processFile: " + file);
					monitor.subTask(file.getName());

					// ファイルを処理 (テンプレート・メソッド)
					final T model = processFile(file);

					models.add(model);
				} catch (final SAXException e) {
					CatStatPlugin.log(e);
				} catch (final IOException e) {
					CatStatPlugin.log(e);
				}
			}
		}
	}

	protected void setViewInput(ExecutionEvent event, List<T> models) {
		try {
			final IWorkbenchWindow window = HandlerUtil.getActiveWorkbenchWindow(event);
			final AbstractStatView view = (AbstractStatView) window.getActivePage().showView(getViewId());
			view.setInput(models);
		} catch (final PartInitException e) {
			CatStatPlugin.log(e);
		}
	}

	protected String getViewId() {
		final String handlerName = getClass().getName();
		return handlerName.replace(".handler.", ".views.").replace(".Show", ".").replace("Handler", "View");
	}

	protected String[] splitWord(String source) {
		return source.split("\\s+");
	}
}
